use crate::prelude::*;
use presentation::wsi::windowsystemintegration::WindowSystemIntegration;

use anyhow::Result;
use std::path::Path;

pub struct WindowConfig<'a> {
    wsi: &'a WindowSystemIntegration,
}

impl<'a> WindowConfig<'a> {
    pub(crate) fn new(wsi: &'a WindowSystemIntegration) -> Self {
        WindowConfig { wsi }
    }

    pub fn set_cursor<T: AsRef<Path>>(&self, bmp: T) -> Result<()> {
        self.wsi.set_cursor(bmp)?;

        Ok(())
    }

    pub fn toggle_fullscreen(&self) -> Result<()> {
        self.wsi.set_fullscreen(!self.wsi.is_fullscreen())?;

        Ok(())
    }

    pub fn is_fullscreen(&self) -> bool {
        self.wsi.is_fullscreen()
    }

    pub fn set_icon<T: AsRef<Path>>(&self, bmp: T) -> Result<()> {
        self.wsi.set_icon(bmp)?;

        Ok(())
    }

    pub fn set_opacity(&self, opacity: f32) -> Result<()> {
        self.wsi.set_opacity(opacity)?;

        Ok(())
    }

    pub fn show_simple_info_box(&self, title: &str, message: &str) -> Result<()> {
        self.wsi.show_simple_info_box(title, message)?;

        Ok(())
    }

    pub fn show_simple_warning_box(&self, title: &str, message: &str) -> Result<()> {
        self.wsi.show_simple_warning_box(title, message)?;

        Ok(())
    }

    pub fn show_simple_error_box(&self, title: &str, message: &str) -> Result<()> {
        self.wsi.show_simple_error_box(title, message)?;

        Ok(())
    }

    pub fn displays(&self) -> &[Display] {
        self.wsi.displays()
    }

    pub fn clipboard_content(&self) -> Result<Option<String>> {
        self.wsi.clipboard_content()
    }
}
