use anyhow::Result;

use crate::prelude::*;

pub trait CheckEntity<T> {
    fn check_entity<Filter>(object: &EntityObject) -> bool
    where
        Filter: CheckFilter;
}

pub trait SceneQuery<'a, T, F> {
    fn query<Filter>(&'a self, f: F) -> Result<()>
    where
        Filter: CheckFilter;
}

macro_rules! impl_add_query {
    ( $($var: ident$(,)?)+ ) => {
        impl< $( $var, )+ > CheckEntity< ( $( $var, )+ ) > for Scene
        where
            $(
                $var: EntityComponent + ComponentDebug,
            )+
        {
            fn check_entity<Filter>(entity: &EntityObject) -> bool
            where
                Filter: CheckFilter,
            {
                let filter = Filter::default();

                $(
                    filter.verify_dedup::<$var>();
                )+

                $(
                    if !entity.components.contains::<$var>() {
                        return false;
                    }
                )+

                if !filter.check(entity) {
                    return false;
                }

                true
            }
        }

        impl<'a, $( $var, )+ F > SceneQuery< 'a, ( $( &'a $var, )+ ), F > for Scene
        where
            $(
                $var: EntityComponent + ComponentDebug,
            )+
            Self: CheckEntity<( $( $var, )+ )>,
            F: FnMut( Entity, $( &'a $var, )+ ) -> Result<()>,
        {
            // fn query<Filter, Func>(&'a self, _query: Query<( $( &'a $var, )+ ), Filter>) -> impl Iterator<Item = (Entity, ($( &'a $var, )+))>
            // where
            //     Filter: CheckFilter,
            //     Query< ( $( &'a $var, )+ ), Filter >: CheckEntity< 'a, ( $( &'a $var, )+ ), Filter >,
            //     F: FnMut(Entity, T) -> R,
            // {
            //     self.entities.iter().filter_map(|(entity, object)| {
            //         Query::filter(*entity, object)
            //     })
            // }

            fn query<Filter>(&'a self, mut f: F) -> Result<()>
            where
                Filter: CheckFilter,
            {
                paste::paste! {
                    self.entities.iter().filter_map(|(entity, object)| {
                        if Self::check_entity::<Filter>(object) {
                            $(
                                let [< $var:lower >] = object.get_component::<$var>().unwrap();
                            )+

                            Some(( entity, $( [< $var:lower >], )+ ))
                        } else {
                            None
                        }
                    })
                    .try_for_each(|( entity, $( [< $var:lower >], )+ )| f( *entity, $( [< $var:lower >], )+ ))
                }
            }
        }


        impl< 'a, $( $var, )+ > CheckEntity< ( $( $var, )+ ) > for Entities<'a>
        where
            $(
                $var: EntityComponent + ComponentDebug,
            )+
        {
            fn check_entity<Filter>(entity: &EntityObject) -> bool
            where
                Filter: CheckFilter,
            {
                let filter = Filter::default();

                $(
                    filter.verify_dedup::<$var>();
                )+

                $(
                    if !entity.components.contains::<$var>() {
                        return false;
                    }
                )+

                if !filter.check(entity) {
                    return false;
                }

                true
            }
        }

        impl<'a, $( $var, )+ F > SceneQuery<'a, ( $( &'a $var, )+ ), F > for Entities<'a>
        where
            $(
                $var: EntityComponent + ComponentDebug,
            )+
            Self: CheckEntity<( $( $var, )+ )>,
            F: FnMut(Entity, $( &'a $var, )+) -> Result<()>,
        {
            fn query<Filter>(&'a self, mut f: F) -> Result<()>
            where
                Filter: CheckFilter,
            {
                paste::paste! {
                    self.entities.iter().filter_map(|(entity, object)| {
                        if Self::check_entity::<Filter>(object) {
                            $(
                                let [< $var:lower >] = object.get_component::<$var>().unwrap();
                            )+

                            Some(( entity, $( [< $var:lower >], )+ ))
                        } else {
                            None
                        }
                    })
                    .try_for_each(|( entity, $( [< $var:lower >], )+ )| f( *entity, $( [< $var:lower >], )+ ))
                }
            }
        }
    };
}

impl_add_query!(Q);
impl_add_query!(Q, R);
impl_add_query!(Q, R, S);
impl_add_query!(Q, R, S, T);
impl_add_query!(Q, R, S, T, U);
impl_add_query!(Q, R, S, T, U, V);
impl_add_query!(Q, R, S, T, U, V, W);
impl_add_query!(Q, R, S, T, U, V, W, X);
impl_add_query!(Q, R, S, T, U, V, W, X, Y);
impl_add_query!(Q, R, S, T, U, V, W, X, Y, Z);
