pub mod hardwareraytracer;
pub mod raytracer;
pub mod softwareraytracer;

mod animator;
mod light;
mod raytraceshader;

use crate::prelude::*;
use animator::VertexData;
use anyhow::Result;

use std::sync::{Arc, Mutex};

pub trait RenderingCore: Sized + Sync + Send {
    fn new(
        device: &Arc<Device>,
        queue: &Arc<Mutex<Queue>>,
        raytracing_info: RaytracingInfo,
        view: &View,
        image: &Arc<Image>,
        position_image: &Arc<Image>,
        descriptors: Vec<&dyn VkHandle<VkDescriptorSetLayout>>,
    ) -> Result<Self>;

    fn recompile(&mut self, descriptors: &[&dyn VkHandle<VkDescriptorSetLayout>]) -> Result<()>;

    fn check(device: &Arc<Device>) -> bool;

    fn resize(&mut self, image: &Arc<Image>, positions: &Arc<Image>) -> Result<()>;
    fn invalidate(&mut self);
    fn process(
        &mut self,
        buffer_recorder: &mut CommandBufferRecorder<'_>,
        descriptors: &[&Arc<DescriptorSet>],
        scene_buffer: &Arc<Buffer<VertexData>>,
        index: usize,
    ) -> Result<()>;
}
