use crate::*;

use anyhow::Result;
use cgmath::{Deg, Vector3};
use rpg_components::{components::abilityloader::AbilityLoader, damage_type::DamageType};

pub struct ArcInfo {
    pub radius: f32,
    pub base_damage: u32,
    pub damage_type: DamageType,
    pub owner: Entity,
}

pub struct OnHitParticles {
    pub particle_system_info: ParticleSystemInfo,
    pub collision_sound: String,

    pub position: Vector3<f32>,
    pub offset: Vector3<f32>,

    pub arc_info: Option<ArcInfo>,
}

impl OnHitParticles {
    pub fn setup(game_handle: &GameHandle, scene: &mut Scene) {
        scene.register_event::<Self>();

        scene.add_event_reader({
            let game_handle = game_handle.clone();

            move |scene, on_hit_particles: &Self| on_hit_particles.spawn(scene, &game_handle)
        });
    }

    fn spawn(&self, scene: &mut SceneContents<'_>, game_handle: &GameHandle) -> Result<()> {
        let game = game_handle.upgrade();

        let mut entity = AbilityLoader::create_on_hit_particles(
            game.engine(),
            self.particle_system_info.clone(),
            &self.collision_sound,
            self.position,
            self.offset,
            scene.particle_system_vulkan_objects(),
            |s| game.build_data_path(s),
        )?;

        if let Some(arc_info) = &self.arc_info {
            entity.insert_component(AreaOfEffectArc::new(
                Deg(180.0),
                arc_info.radius,
                arc_info.base_damage,
                arc_info.damage_type,
                arc_info.owner,
            ));

            Faction::copy(
                &mut scene.entity_mut(arc_info.owner)?.multi_mut(),
                &mut entity,
            );
        }

        scene.add_entity(entity)?;

        Ok(())
    }
}
