use rpg_components::components::abilityloader::AbilityLoader;

use crate::*;

#[derive(Debug, Clone, Copy)]
pub enum ParticleSpawnExecutor {
    AoE,
    Projectile,
}

pub struct ParticleSpawn {
    pub entity: Entity,
    pub info: AbilityLoader,
    pub executor: ParticleSpawnExecutor,
}

impl ParticleSpawn {
    pub fn setup_event(game_handle: &GameHandle, scene: &mut Scene) {
        scene.register_event::<Self>();

        scene.add_event_reader({
            let game_handle = game_handle.clone();

            move |scene, particle_spawn: &Self| {
                let particle_system_vulkan_objects =
                    unsafe { remove_life_time_mut(scene) }.particle_system_vulkan_objects();

                let particle_system = {
                    let owner = scene.entity_mut(particle_spawn.entity)?;

                    let mut multi_mut = owner.multi_mut();
                    let draw = multi_mut.get::<Draw>()?;

                    match particle_spawn.executor {
                        ParticleSpawnExecutor::AoE => {
                            let aoe = multi_mut.get::<AreaOfEffect>()?;

                            aoe.create_particle_effect(
                                &game_handle,
                                &particle_spawn.info,
                                draw,
                                particle_system_vulkan_objects,
                            )?
                        }
                        ParticleSpawnExecutor::Projectile => Projectile::create_particles(
                            &particle_spawn.info,
                            &game_handle,
                            draw,
                            particle_system_vulkan_objects,
                        )?,
                    }
                };

                scene.insert_component(particle_spawn.entity, particle_system)
            }
        });
    }
}
