use engine::prelude::*;

#[derive(Debug, Clone, Copy, PartialEq, Eq)]
pub enum Faction {
    Player,
    Friendly,
    NPC,
}

impl Faction {
    pub fn copy(source: &mut MultiMut<'_>, destination: &mut EntityObject) {
        if source.get::<FactionPlayer>().is_ok() {
            destination.insert_component(FactionPlayer);
        } else if source.get::<FactionNPC>().is_ok() {
            destination.insert_component(FactionNPC);
        } else {
            unreachable!("missing component!");
        }
    }

    pub fn get(obj: &mut MultiMut<'_>) -> Option<Self> {
        if obj.get::<FactionPlayer>().is_ok() {
            Some(Self::Player)
        } else if obj.get::<FactionFriendly>().is_ok() {
            Some(Self::Friendly)
        } else if obj.get::<FactionNPC>().is_ok() {
            Some(Self::NPC)
        } else {
            None
        }
    }

    pub fn check_collision(self, other: Option<Self>) -> bool {
        match other {
            Some(collider_faction) => {
                // same faction or friendly NPCs are ignored
                if self == collider_faction || collider_faction == Self::Friendly {
                    return false;
                }

                true
            }
            None => false,
        }
    }
}

#[derive(Debug, Clone, Copy, Eq, PartialEq)]
pub struct FactionPlayer;

impl EntityComponent for FactionPlayer {
    fn name(&self) -> &str {
        Self::debug_name()
    }
}

impl ComponentDebug for FactionPlayer {
    fn debug_name() -> &'static str {
        "FactionPlayer"
    }
}

#[derive(Debug, Clone, Copy, Eq, PartialEq)]
pub struct FactionFriendly;

impl EntityComponent for FactionFriendly {
    fn name(&self) -> &str {
        Self::debug_name()
    }
}

impl ComponentDebug for FactionFriendly {
    fn debug_name() -> &'static str {
        "FactionFriendly"
    }
}

#[derive(Debug, Clone, Copy, Eq, PartialEq)]
pub struct FactionNPC;

impl EntityComponent for FactionNPC {
    fn name(&self) -> &str {
        Self::debug_name()
    }
}

impl ComponentDebug for FactionNPC {
    fn debug_name() -> &'static str {
        "FactionNPC"
    }
}
